:- module(kategorien,[kategorie/1]).

% --- Wertebereiche der Art- und Formmerkmale -------

person(X) :- (X = 1 ; X = 2 ; X = 3). 
tempus(X) :- (X = praes ; X = praet). 
modus(X) :- (X = ind ; X = konj).

genus(Gen) :-
    (Gen = fem ; Gen = mask ; Gen = neut).
numerus(X) :-
    (X = sg ; X = pl).
kasus(X) :-
    (X = nom ; X = gen ; X = dat ; X = akk).

% --- Kategorie(Artmerkmale,Formmerkmale) -----------

kategorie(en([Gen],[sg,Kas])) :-
    genus(Gen), kasus(Kas).

kategorie(n([Gen],[Num,Kas])) :-
    genus(Gen), numerus(Num), kasus(Kas).

kategorie(rn([Gen],[Num,Kas])) :-
    genus(Gen), numerus(Num), kasus(Kas).
kategorie(pron([Def],[Gen,Num,Kas])) :-
    (Def = def ; Def = rel ; Def = qu), % er, der, wer
    genus(Gen), numerus(Num), kasus(Kas).
    
kategorie(det([Def],[Gen,Num,Kas])) :-
    % ein/der,welcher,jeder   % TODO quant? ggf: def/qu
    member(Def,[indef,def,qu,quant]),   
    genus(Gen), numerus(Num), kasus(Kas).

kategorie(poss([Def],[Gen,Num,Kas])) :-
    member(Def,[def,rel,qu]), % sein,dessen,wessen
    genus(Gen), numerus(Num), kasus(Kas).

kategorie(np([Def,Pers,Gen,Num],[Kas])) :-  
    definitheit(np,Def), % Num inhaerent
    Pers = 3, % TODO für Dialoge: 1.+2.Person
    genus(Gen), numerus(Num), kasus(Kas).

definitheit(np,Def) :-
    ( Def = indef ; Def = def ; Def = qu
    ; Def = quant ; Def = refl ; Def = rezi 
    ; Def = rel(Gen,Num), genus(Gen), numerus(Num)
    ).

kategorie(v([nom],[Pers,Num,Temp,Mod])) :-
    Pers = 3,      % TODO: 1.2 Person fuer Dialoge
    numerus(Num), tempus(Temp), modus(Mod). % Mod := ind

kategorie(v([nom,akk],[Pers,Num,Temp,Mod])) :-
    Pers = 3,      % TODO: 1.2 Person fuer Dialoge
    numerus(Num), tempus(Temp), modus(Mod). % Mod := ind

kategorie(v([H],[Pers,Num,Temp,Mod])) :-
    hilfsverb(H),
    Pers = 3, numerus(Num), tempus(Temp), modus(Mod).
hilfsverb(H) :-
    (H = sein ; H = werden).  % TODO: Modalverben

kategorie(s([Def],[Temp,Mod,Vst])) :- % einfache Saetze
    definitheit(s,Def),
    tempus(Temp), % ggf. ; Temp = perf
    modus(Mod),
    verbstellung(Vst).

definitheit(s,Def) :-
    (Def = def ; Def = qu
    ; Def = rel(Gen,Num), genus(Gen),numerus(Num)).
verbstellung(Vst) :-
    (Vst = ve ; Vst = vz ; Vst = vl).

% TODO: Kategorie komplexer Saetze: welche Merkmale?
% kategorie(s([Def],[])) :-  % koordinierte Saetze

