% eval(+Belegung,+Term,-Objekt)
eval([(Var,Obj)|Env],X,Val) :-
    var(X),
    !, (Var == X -> Val = Obj ; eval(Env,X,Val)).
eval([],X,_) :-
    var(X),
    !, format('\nFehler: unbelegte Variable').
eval(_Env,C,Val) :-
    functor(C,C,0), !, Val = C.

% evals(+Belegung,+Terme,-Objekte)
evals(Env,[Term|Terms],[Val|Vals]) :-
    eval(Env,Term,Val), evals(Env,Terms,Vals).
evals(_Env,[],[]).
    
eval(_Env,anzahl(X,_Restr,_Formel),_N) :-
    var(X),!,fail. % nur getypte Ausdrücke auswerten

eval(Env,anzahl(X:Ty,Restr,Formel),N) :-
    var(X),!,
    findall(O,(objekt(O:Ty),
               eval([(X,O)|Env],(Restr & Formel))),Os),
    sort(Os,Objs), length(Objs,N).

eval(Env,anzahl((X,Y):TyX*TyY,Restr,Formel),N) :-
    var(X),var(Y),!,
    findall((A,B),(objekt(A:TyX),objekt(B:TyY),
               eval([(X,A),(Y,B)|Env],(Restr & Formel))),
            ABs), sort(ABs,Pairs), length(Pairs,N).
% eval(+Belegung,+Formel) 

% aussagenlogische Verbindungen:
:- op(500,yfx,&), op(600,yfx,'\/'),
    op(600,xfx,'=>'), op(600,xfx,'<=>').

eval(Env,(F & G)) :- !, eval(Env,F) , eval(Env,G).
eval(Env,(F \/ G)) :- !, (eval(Env,F), ! ; eval(Env,G)). 
eval(Env,(F => G)) :-  !, eval(Env,(neg(F) \/ G)).
eval(Env,(F <=> G)) :- !, eval(Env,(F => G) & (G => F)).
eval(Env,neg(F)) :- !, (eval(Env,F) -> fail ; true).

eval(_Env,true) :- !.

% Prädikatenlogische Quantoren:
eval(Env,all(Var,Restr,Fml)) :-
    !, eval(Env, neg(ex(Var,Restr,neg(Fml)))).

eval(Env,ex(Var,Restr,Fml)) :-
    var(Var), !, % ungetypt
    datenbank:objekt(Obj:_Ty), % suche in der Datenbank
    eval([(Var,Obj)|Env], Restr),
    eval([(Var,Obj)|Env], Fml), !. % nur erste Lösung

% Verallgemeinerte Quantoren: (TODO)
% all, ex, most, iota

eval(_Env,most(X,_Restr,_Formel)) :-
    var(X), !, fail. % keine Semantik ungetypter Aussagen 
eval(Env,most(X:Ty,Restr,Formel)) :-
    var(X), !,
    eval(Env,anzahl(X:Ty,Restr,Formel),NPos),
    eval(Env,anzahl(X:Ty,Restr,neg(Formel)),NNeg),
    NPos > NNeg.
eval(Env,most((X,Y):(TyX * TyY),Restr,Formel)) :-
    eval(Env,anzahl((X,Y):(TyX*TyY),Restr,Formel),NPos),
    eval(Env,anzahl((X,Y):(TyX*TyY),Restr,neg(Formel)),NNeg),
    NPos > NNeg.

eval(Env,qu((X,Y):(TyX * TyY),Restr,Formel), Paare) :-
    !, findall((A,B),(objekt(A:TyX), objekt(B:TyY),
               eval([(X,A),(Y,B)|Env],(Restr & Formel))),
               ABs),
    sort(ABs,Paare).
eval(Env,(X < Y))  :-
    !, evals(Env,[X,Y],[VX,VY]), VX < VY.     
eval(Env,(X =< Y))  :-
    !, evals(Env,[X,Y],[VX,VY]), VX =< VY.     
eval(Env,eq(X,Y))  :-
    !, evals(Env,[X,Y],[VX,VY]), VX == VY.     

% Grundformeln (der Datenbank)
eval(Env,Formel) :-
    functor(Formel,Pn,Arity),
    Arity > 0, !, 
    Formel =.. [Pn|Args],
    evals(Env,Args,Vals),
    EFormel =.. [Pn|Vals], call(EFormel).

% Anzahlaussagen
eval(Env,anzahl(Var,Formel,Zahl)) :-
    !, antworten(Env,qu(Var,true,Formel),Objekte), 
    length(Objekte,Zahl). % N >= Zahl ?

beantworte(PL_Aussage,Wert) :-
    eval([],PL_Aussage) -> Wert = ja ; Wert = nein.

antworten(qu(V,Restr,Formel), As) :-
    var(V), !, 
    findall(O,(objekt(O:_), 
               eval([(V,O)],(Restr & Formel))),Ws),
    sort(Ws,As).
antworten(qu(V:Typ,Restr,Formel), As) :-
    var(V), !,
    findall(O,(objekt(O:Typ), 
               eval([(V,O)],(Restr & Formel))),Ws),
    sort(Ws,As).    
eval(Env,qu((X,Y):(TyX * TyY),Restr,Formel), Paare) :-
    !, findall((A,B),(objekt(A:TyX), objekt(B:TyY),
               eval([(X,A),(Y,B)|Env],(Restr & Formel))),
               ABs),
    sort(ABs,Paare).

