:- module(lambdaDrs,[reduceTerm/2,mergeTerm/2]).
:- op(500,yfx,&), op(600,yfx,'\/'), op(600,xfx,'=>').

normalize(Term,MergedTerm):-
    reduceTerm(Term,ReducedTerm),
    mergeTerm(ReducedTerm,MergedTerm).

reduceTerm(T,Nf) :-
    beta(T,ConT,Tag), 
    ( Tag = chd -> reduceTerm(ConT,Nf) ; Nf = ConT ).        

reduceTerms([T|Ts],[Nf|Nfs]) :-
    reduceTerm(T,Nf), reduceTerms(Ts,Nfs).
reduceTerms([],[]).

% mergeTerm(+Term,-mergedTerm)

mergeTerm(T,Nf) :-
    mergeDrs(T,MT,Tag),    
    ( Tag = chd -> mergeTerm(MT,Nf) ; Nf = MT ).

mergeTerms([T|Ts],[Nf|Nfs],Tag) :-
    mergeDrs(T,MT,TagT),
    ( TagT = chd -> mergeTerm(MT,Nf) ; Nf = MT ),
    mergeTerms(Ts,Nfs,TagTs),
    ( TagT = not -> Tag = TagTs ; Tag = chd ).    
mergeTerms([],[],not).

% mergeDrs(+ReducedTerm, -mergedTerm, -geaendert?)

mergeDrs(X, X, not) :-  var(X), !.

mergeDrs(T*S, Mrg, Tag) :-
    !, mergeDrs(T,ConT,TagT),
    ( TagT = chd -> Mrg = ConT*S, Tag = chd
    ; mergeDrs(S,ConS,Tag), Mrg = T*ConS ).

mergeDrs(lam(X,D),lam(X,MD),Tag):-
    !, mergeDrs(D,MD,Tag).

mergeDrs(drs(D,F),drs(D,MF),Tag):-
    !, mergeTerms(F,MF,Tag). 

mergeDrs(neg(D),neg(MD),Tag):-
    !, mergeDrs(D,MD,Tag).

mergeDrs((D1 => D2),Mrg,Tag):-
    !, mergeDrs(D1,MD1,Tag1),
    ( Tag1 = chd -> Mrg = (MD1 => D2), Tag = chd
    ; mergeDrs(D2,MD2,Tag),Mrg = (D1 => MD2)).
        
mergeDrs(or(D1,D2),Mrg,Tag):- % TODO: \/ statt or
    !, mergeDrs(D1,MD1,Tag1),
    ( Tag1 = chd -> Mrg = or(MD1,D2), Tag = chd
    ; mergeDrs(D2,MD2,Tag),Mrg = or(D1,MD2)).
        
mergeDrs((drs(R1,F1) + drs(R2,F2)),drs(R,F),chd):-
    !, append(R2,R1,R), append(F2,F1,F). % (Wu)
    % !, append(R1,R2,R), append(F1,F2,F). % (HL)

mergeDrs((D1 + D2), Mrg, Tag):-
    !, mergeDrs(D1,MD1,Tag1),
    ( Tag1 = chd -> Mrg = (MD1 + D2), Tag = chd
    ; mergeDrs(D2,MD2,Tag), Mrg = (D1 + MD2) ).

mergeDrs(F,F,not):-!. % einfache Bedingung
        
