:- module(resolveDrs,[]).

resolve(SemS,SemD,Sem) :- 
    drs:type([],SemS,GetyptS,_TypS),
    reverse_drs(SemD,SemDrev),
    resolve_drs([seq(SemDrev,GetyptS)],[SemRev]),
    reverse_drs(SemRev,Sem).

% :- debug(resolve).  % Pronomenauflösung einschalten.
reverse_drs(drs(R,F),drs(Rrev,Frev)):-
    reverse(R,Rrev),
    reverse(F,Frev).

% resolve_drs(+DRSen, -resolvedDRSen)
resolve_drs([drs(R,F)|Ds1],Ds2):-
    resolve_fml(F,[drs(R,[])|Ds1],Ds2).

/*
resolve_drs([seq(D,S)],[drs(R,F)]):-
    resolve_drs([S,D],[drs(RS,FS),drs(RD,FD)]),
    append(RS,RD,R),
    append(FS,FD,F).
*/
resolve_drs([seq(D,S)|Ds1],[drs(R,F)|Ds3]):-
    resolve_drs([D|Ds1],Ds2),
    resolve_drs([S|Ds2],[drs(RS,FS),drs(RD,FD)|Ds3]),
    append(RS,RD,R),
    append(FS,FD,F).

% resolve_anp(+Ref,+Gen,+Num,+DRSen)
resolve_anp(Ref,Gen,Num,Ds):-
    member(drs(_Refs,Fs),Ds),
    member(ant(Ref,Gen,Num),Fs).

delete_ref(X,[R|Rs],NewRs) :- % entferne aufgelösten
    (X==R -> NewRs = Rs       % Referenten
    ; delete_ref(X,Rs,RsX),NewRs = [R|RsX]).
% delete_ref(_X,[],[]). % (HL) Doppelloesung verhindern!
        
% resolve_fml(+Fml,[?zwischenDRS|erreichbareDRSen],-resolvedDRSen)
resolve_fml([anposs(Ref,Gen,Num)|Fmls],[drs(R,F)|Ds1],Ds2):-  % HL
    !,(resolve_anp(Ref,Gen,Num,[drs(R,F)|Ds1]) % erst im selben satz
      -> ( 
           delete_ref(Ref,R,NewR),
           resolve_fml(Fmls,[drs(NewR,F)|Ds1],Ds2)
         )
      ; (                       % write('"resolve anaphern failed."'),
            resolve_fml(Fmls,[drs(R,[anposs(Ref,Gen,Num)|F])|Ds1],Ds2)
        )).
% why(resolve_fml([anposs(X, fem, sg)], [drs([], [])], [drs(_G3902, _G3903)])).

% resolve_fml([anp(Ref,Gen,Num)|Fmls],[drs(R,F)|Ds1],Ds2):-
%       !,(resolve_anp(Ref,Gen,Num,Ds1))
%       -> ( 
%          delete_ref(Ref,R,NewR),
%          resolve_fml(Fmls,[drs(NewR,F)|Ds1],Ds2)
%          )
%         ; (% write('"\nresolve anaphern failed.\n\n"'),
%       resolve_fml(Fmls,[drs(R,[anp(Ref,Gen,Num)|F])|Ds1],Ds2)).
% resolve_fml(+Fmls,[?resultDRS|accessDRSen],-resolvedDRSen)

resolve_fml([anp(Ref,Gen,Num)|Fmls],
            [drs(R,F)|Ds1], Ds2) :-
    !,
    ( resolve_anp(Ref,Gen,Num,Ds1),
        delete_ref(Ref,R,NewR),
        NewD = drs(NewR,F)
    ; NewD = drs(R,[anp(Ref,Gen,Num)|F])        
    ),
    resolve_fml(Fmls,[NewD|Ds1],Ds2).

resolve_fml([neg(D1)|Fmls],Ds1,Ds3):-
    !, resolve_drs([D1|Ds1],[D2,drs(R,F)|Ds2]),
    resolve_fml(Fmls,[drs(R,[neg(D2)|F])|Ds2],Ds3).

resolve_fml([(D1 => D2)|Fmls],Ds1,Ds4):-
    !, resolve_drs([D1|Ds1],Ds2),
    resolve_drs([D2|Ds2],[D4,D3,drs(R,F)|Ds3]),
    resolve_fml(Fmls,[drs(R,[(D3 => D4)|F])|Ds3],Ds4).

resolve_fml([or(D1,D2)|Fmls],Ds1,Ds4):-
    !, resolve_drs([D1|Ds1],[D3|Ds2]),
    resolve_drs([D2|Ds2],[D4,drs(R,F)|Ds3]),
    resolve_fml(Fmls,[drs(R,[or(D3,D4)|F])|Ds3],Ds4).

resolve_fml([Fml|Fmls],[drs(R,F)|Ds1],Ds2):-
    !, resolve_fml(Fmls,[drs(R,[Fml|F])|Ds1],Ds2).

resolve_fml([],[drs(R,F)|Ds],[drs(R,Frvs)|Ds]):-
    !, reverse(F,Frvs).

