:- op(500,yfx,&), op(600,yfx,'\/'),
    op(600,xfx,'=>'), op(600,xfx,'<=>').

kleinschreibung(Wort,Klein) :-    
    downcase_atom(Wort,Klein).

sem([en(Art,Form), [Vollform]],LamTerm) :-
    wort(Stammform, en(Art,Form), Vollform),
    kleinschreibung(Stammform,LamTerm).

sem([rn(Art,Form), [Vollform]],LamTerm) :-
    wort(Stammform, rn(Art,Form), Vollform),
    kleinschreibung(Stammform,Stamm),
    Formel =.. [Stamm,X,Y],
    LamTerm = lam(X,lam(Y,Formel)).

sem([v([nom,akk],Form),[Vollform]],LamTerm) :-
    kleinschreibung(Vollform,VollformKl),
    wort(Stammform,v([nom,akk],Form),VollformKl),
    Formel =.. [Stammform,X,Y],
    LamTerm = lam(X,lam(Y,Formel)).

sem([n(Art,Form),[Vollform]],LamTerm) :-
    wort(Stammform, n(Art,Form), Vollform),
    kleinschreibung(Stammform,Stamm),
    Formel =.. [Stamm,X],
    LamTerm = lam(X,Formel).
   
sem([n(Art,Form),[rn(Art,Form),[Vollform]]],LamTerm) :-
    sem([rn(Art,Form),[Vollform]],SemRN),
    normalize(SemRN * X * Y, Formel),
    LamTerm = lam(X,ex(Y,true,Formel)).
   
sem([det([indef],[_,sg,_]),_Vollform],
    lam(N,lam(P,ex(X, N*X, P*X)))).     % ex/3

sem([det([def], [_,sg,_]),_Vollform],
    lam(N,lam(P, iota(X, N*X, P*X)))).  % iota/3

sem([det([quant],[_,sg,_]),[Vollform]],
    lam(N,lam(P,all(X, N*X, P*X)))) :-  % all/3
    concat_atom(['jede'|_],Vollform).

sem([det([def],[_,pl,_]),[Vollform]],
    lam(N,lam(P, anzahl(X,N*X & P*X,Zahl)))) :-
    anzahl(Vollform,Zahl), 2 =< Zahl, !.

sem([det([qu], _Form),[Vollform]],
    lam(N,lam(P,qu(X, N*X, P*X)))) :-
    concat_atom(['welche'|_],Vollform).

sem([det([def], [_,pl,_]),_Vollform],
    lam(N,lam(P, iota(X, N*X, P*X)))).  % iota/3

sem([det([quant],[_,pl,_]),[Vollform]],
    lam(N,lam(P,all(X,N*X,P*X)))) :-
    concat_atom(['alle'|_],Vollform).

sem([det([quant],[_,pl,_]),[_DefArt],[meisten]],
    lam(N,lam(P,most(X,N*X,P*X)))).

sem([a([als(nom)],[komp]),[kleiner]],
    lam(X,lam(Y,(X < Y)))). 
sem([a([als(nom)],[komp]),[größer]],
    lam(X,lam(Y,(Y < X)))). 

sem([pron([rel],_Form),_Rel], lam(P,lam(X,P*X))).
sem([pron([qu],_Form),[Wer]], lam(P,qu(X,true,P*X))) :-
    ( Wer = 'wer' ; Wer = wen ).

sem([poss([rel],[_Gen,_Num,gen]),_Poss],
    lam(RN,lam(P,lam(Y,ex(X, RN*X*Y, P*X))))).
    % nur bei RNsg 
sem([np([_,3,_,_],[_]),
       Det, [n(Art,Form),Vf]], SemNP) :-
    Vf \= km, sem(Det,SemDet),
    sem([n(Art,Form),Vf],SemN),
    normalize(SemDet * SemN,SemNP).

sem([np([def,3,Gen,Num],[Kas]),
       [en([Gen],[Num,Kas]),EN]], SemNP) :-
    sem([en([Gen],[Num,Kas]),EN],SemEN),
    SemNP = lam(P,P*SemEN).
sem([np([def,3,Gen,Num],[Kas]),
     [det([def],[Gen,Num,Kas]),_],
     [en([Gen2],[Num,nom]),EN]], SemNP) :-
    sem([en([Gen2],[Num,nom]),EN], SemEN),
    SemNP = lam(P, P*SemEN).

sem([np(_Art,_Form),[pron([rel],Form),VF]],SemNP) :-
    sem([pron([rel],Form),VF],SemNP).

sem([np(_Art,_Form),[pron([qu],Form),VF]],SemNP) :-
    sem([pron([qu],Form),VF],SemNP).

sem([np([Def,3,Gen,Num],[Kas]),
     [det([Def],[Gen,Num,Kas])|Det],
     [rn([Gen],[Num,Kas]),RN],
     [np(Art,[gen])|Const]],SemNP) :-
    sem([det([Def],[Gen,Num,Kas])|Det],SemDet),
    sem([rn([Gen],[Num,Kas]),RN],SemRN),
    sem([np(Art,[gen])|Const],SemNPGen),
    normalize(SemDet *
       lam(X, SemNPGen*lam(Y,(SemRN*X*Y))), SemNP).

sem([s([_Def],[_,_,vz]),
      NP1, [v([nom,akk],[3,Num,_,ind]),V], NP2],
    SemS)
:-  not(debugging(typisieren)), % ohne Typen!
    sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],[3,Num,_,ind]),V], SemV),
    NP1 = [np([_,3,_,Num1],[Kas1])|_Konst], 
    (Kas1 = nom, Num1 = Num,
        Sem = SemNP1 * lam(X,SemNP2 *
                          lam(Y,SemV * X * Y))
    ; Kas1 = akk,
        Sem = SemNP1 * lam(Y,SemNP2 *
                          lam(X,SemV * X * Y))
    ), normalize(Sem,SemS).
       
sem([s([rel(_Gen,Num)],_Form),
           NP1, NP2,
           [v([nom, akk], FormV),V]], SemRelS) 
:-  not(debugging(typisieren)), % ohne Typen!
    NP1 = [np([_,3,_,Num1],[Kas1])|_], 
    sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],FormV),V], SemV),
    (Kas1 = nom, Num1 = Num,
        Sem = SemNP1 * lam(X,SemNP2 *
                          lam(Y,SemV * X * Y))
    ; Kas1 = akk,
        Sem = SemNP1 * lam(Y,SemNP2 *
                          lam(X,SemV * X * Y))
    ), normalize(Sem,SemRelS).
       
sem([s([_Def],[_,_,ve]),
      [v([nom,akk],[3,Num,_,ind]),V], NP1, NP2],
    SemS)
:-  sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],[3,Num,_,ind]),V], SemV),
    NP1 = [np([_,3,_,Num1],[Kas1])|_Konst], 
    (Kas1 = nom, Num1 = Num,
        Sem = SemNP1 * lam(X,SemNP2 *
                          lam(Y,SemV * X * Y))
    ; Kas1 = akk,
        Sem = SemNP1 * lam(Y,SemNP2 *
                          lam(X,SemV * X * Y))
    ), normalize(Sem,SemS).
       
sem([np([_Def,3,Gen,Num],[Kas]),
     Det, [n([Gen],[Num,Kas]),N],
     [s([rel(Gen,Num)],[Tmp,ind,vl])|Const]],
    SemNP) :-
    sem(Det,SemDet),
    sem([n([Gen],[Num,Kas]),N],SemN),
    sem([s([rel(Gen,Num)],[Tmp,ind,vl])|Const],SemS),
    normalize(SemDet * lam(X, SemN*X & SemS*X),
              SemNP).
% Paar-Lesarten der Pluraldeterminationen (Typen!)
sem([det([quant],[_,pl,_]),[Vollform]],
    lam(N,lam(P,all((X,Y),N*X & N*Y,P*X*Y)))) :-
    concat_atom(['alle'|_],Vollform).        

sem([det([quant],[_,pl,_]),[Vollform]],
    lam(N,lam(P,all((X,Y),N*X*Y,P*X*Y)))) :-
    concat_atom(['alle'|_],Vollform).

sem([det([quant],[_,pl,_]),[_DefArt],[meisten]],
    lam(N,lam(P,most((X,Y),N*X & N*Y,P*X*Y)))).

sem([det([quant],[_,pl,_]),[_DefArt],[meisten]],
    lam(N,lam(P,most((X,Y),N*X*Y,P*X*Y)))).

sem([det([quant],[_,pl,_]),[die]],
    lam(N,lam(P,iota((X,Y),N*X & N*Y,P*X*Y)))).

sem([det([qu],[_,pl,_]),[welche]],
    lam(N,lam(P,qu((X,Y),N*X & N*Y,P*X*Y)))).

sem([det([qu],[_,pl,_]),[welche]],
    lam(N,lam(P,qu((X,Y),N*X*Y,P*X*Y)))).

sem([np([_,3,_,pl],[Kas]),
       Det, [n(Art,Form),[rn(Art,Form),Vf]]], SemNP) :-
    Form = [pl,Kas], % nur für Plural
    sem(Det,SemDet),
    sem([rn(Art,Form),Vf],SemRN),
    normalize(SemDet * SemRN,SemNP).

sem([pron([refl],_Form),[sich]], lam(P,lam(X,P*X*X))).
sem([np([refl,3,Gen,Num],[Kas]), 
     [pron([refl],[Gen,Num,Kas]),VF]], SemPron) :-
    sem([pron([refl],[Gen,Num,Kas]),VF], SemPron).

sem([np([rezi,3,Gen,Num],[Kas]), 
     [pron([rezi],[Gen,Num,Kas]),VF]], SemPron) :-
    sem([pron([rezi],[Gen,Num,Kas]),VF], SemPron).

sem([pron([rezi],_Form),[einander]], 
    lam(P,lam(X,lam(Y,P*X*Y & P*Y*X)))).

sem([s([_Def],[_,_,vz]),
      NP1, [v([nom,akk],[3,Num,_,ind]),V], NP2],
    SemS)
:-  sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],[3,Num,_,ind]),V], SemV),
    NP1 = [np([_,3,_,Num1],[nom])|_Konst1], 
    NP2 = [np([Def,3,_,_Num2],[akk])|_Konst2], 
    ( Num1 = Num, Def=refl, 
        Sem = SemNP1 * (SemNP2 * SemV)
    ; Num1 = Num, Def=rezi, 
        Sem = SemNP1 * (SemNP2 * SemV)
    ), normalize(Sem,SemS).

sem([s([_Def],[_,_,ve]),
      [v([nom,akk],[3,Num,_,ind]),V], NP1, NP2],
    SemS)
:-  sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],[3,Num,_,ind]),V], SemV),
    NP1 = [np([_,3,_,Num1],[nom])|_Konst1], 
    NP2 = [np([Def,3,_,_Num2],[akk])|_Konst2], 
    ( Num1 = Num, Def=refl, 
        Sem = SemNP1 * (SemNP2 * SemV)
    ; Num1 = Num, Def=rezi, 
        Sem = SemNP1 * (SemNP2 * SemV)
    ), normalize(Sem,SemS).

