:- module(drs,[]).

% sem/2 mit lambda-DRS-Termen
:- op(500,yfx,&), op(600,yfx,'\/'), op(600,xfx,'=>').

kleinschreibung(Wort,Klein) :-    
    downcase_atom(Wort,Klein).

% EN
sem([en(Art,Form), [Vollform]],LamTerm) :-
    !, wort(Stammform, en(Art,Form), Vollform),
    kleinschreibung(Stammform,EN),
    LamTerm = EN.

% VI 
sem([v([nom],Form),[Vollform]],LamTerm) :-
    !, kleinschreibung(Vollform,VollformKl),
    wort(Stammform,v([nom],Form),VollformKl),
    Formel =.. [Stammform,X],
    LamTerm = lam(X,drs([],[Formel])).

% VT 
sem([v([nom,akk],Form),[Vollform]],LamTerm) :-
    !, kleinschreibung(Vollform,VollformKl),
    wort(Stammform,v([nom,akk],Form),VollformKl),
    Formel =.. [Stammform,X,Y],
    LamTerm = lam(X,lam(Y,drs([],[Formel]))).
   
% CN
sem([n([Gen],[Num,Kas]),[Vollform]],LamTerm) :-
    !, wort(Stammform, n([Gen],[Num,Kas]), Vollform),
    kleinschreibung(Stammform,Stamm),
    Formel =.. [Stamm,X],
    LamTerm = lam(X,drs([],[ant(X,Gen,Num),Formel])).
   
% RN 
sem([rn([Gen],[Num,Kas]), [Vollform]],LamTerm) :-
    !, wort(Stammform, rn([Gen],[Num,Kas]), Vollform),
    kleinschreibung(Stammform,Stamm),
    Formel =.. [Stamm,X,Y],
    LamTerm = lam(X,lam(Y,drs([],[Formel,
                                  ant(X,Gen,Num)]))).

sem([n(Art,Form),[rn(Art,Form),[Vollform]]],LamTerm) :-
    !, sem([rn(Art,Form),[Vollform]],SemRN),
    normalize(SemRN * X * Y, Drs),
    LamTerm = lam(X,drs([Y],[])+Drs).
   
% DET = every % distributive sg-Lesart; 2.pl-Lesart unten
sem([det([quant],_Form),_Vollform],
    lam(N,lam(P,drs([],[((drs([X],[]) + N*X) => P*X)])))). 

% DET = a  
sem([det([indef],_Form),_Vollform], % 2.pl-Lesart unten
    lam(N,lam(P, drs([X],[]) + N*X + P*X))).

% DET = the 
sem([det([def],[_,sg,_]),_Vollform],
    lam(N,lam(P, drs([Y],[(drs([X],[]) + N*X)
                         => drs([],[eq(X,Y)])])
                 + N*Y + P*Y))) :- !.

% DET = alle % Paar-Lesart mit symmetrischem P
sem([det([quant],[_,pl,_]),_Vollform],
    lam(N,lam(P,drs([],[((drs([X,Y],[]) + N*X + N*Y) 
                         => P*X*Y)])))) :- !.
% DET = einige
sem([det([indef],[_,pl,_]),_Vollform],
    lam(N,lam(P, drs([X,Y],[]) + N*X + N*Y + P*X*Y))) :- !.
% DET = die
sem([det([def],[_,pl,_]),_Vollform],
    lam(N,lam(P, drs([Y1,Y2],
                     [(drs([X1,X2],[]) + N*X1 + N*X2)
                      => drs([],[eq(X1,Y1),eq(X2,Y2)])])
                 + N*Y1 + N*Y2 + P*Y1*Y2))) :- !.

% TODO: drs?, sg/pl-Lesarten?
sem([det([qu], _Form),[Vollform]],
    lam(N,lam(P,qu(X,N*X & P*X)))) :-
    concat_atom(['welche'|_],Vollform), !.

sem([det([def],[_,pl,_]),[Vollform]],
    lam(N,lam(P, drs([],[anzahl(X,N*X & P*X,Zahl)])))) :-
    anzahl(Vollform,Zahl), !.

% PRON [per]
sem([pron([per],[Gen,Num,_Kas]),_Pron],
    lam(P, drs([X],[anp(X,Gen,Num)]) + P*X)) :- !.

% TODO
sem([pron([rel],_Form),_Rel], lam(P,lam(X,P*X))) :- !.
sem([pron([qu],_Form),_Pron], lam(P,qu(X,P*X))) :- !.

% POSS (ohne Eindeutigkeitsbedingung) (HL) 
sem([poss([def(Gen2,Num2)],_Form),_Vollform],Sem):-
    !, Sem = lam(RN,lam(P, drs([X,Y],
                               [anposs(Y,Gen2,Num2)])
                           + RN*X*Y + P*X)).
sem([pron([refl],_Form),[sich]],
    lam(P,lam(X, P*X*X))) :- !.

sem([pron([rezi],_Form),[einander]], 
    lam(P,lam(X,lam(Y, P*X*Y + P*Y*X)))) :- !.

% NP = DET+CN 
sem([np([_,3,_,_],[_]),
       Det, [n(Art,Form),Vf]], SemNP) :-
    sem(Det,SemDet), Vf \= km,
    !, sem([n(Art,Form),Vf],SemN),
    normalize(SemDet * SemN,SemNP).

% NP = EN (mit neuem Diskursreferenten)
sem([np([def,3,Gen,Num],[Kas]),
       [en([Gen],[Num,Kas]),EN]], SemNP) :-
    !, sem([en([Gen],[Num,Kas]),EN],SemEN),
    SemNP = lam(P,drs([X],[eq(X,SemEN),ant(X,Gen,Num)])
                  + P*X).
% NP = DET EN 
sem([np([def,3,Gen,Num],[_Kas]),
       [det([def],_),_],
       [en([Gen],[Num,nom]),EN]],
    SemNP) :-
    !, sem([np([def,3,Gen,Num],[nom]),
            [en([Gen],[Num,nom]),EN]], SemNP).

% NP = DET+N+PN 
sem([np([def,3,Gen,Num],[Kas]), 
     [det([def],_),_],
     [n([Gen],[Num,Kas]),N],
     [en([Gen2],[Num,nom]),EN]], SemNP) :-
    !, sem([n([Gen],[Num,Kas]),N],SemN),
    sem([en([Gen2],[Num,nom]),EN],SemEN),
    normalize(lam(P, drs([X],[eq(X,SemEN)])
                     + SemN*X + P*X), SemNP).

% NP = PRON (personal, reflexiv, reziprok, interrogativ)
sem([np([def,3,_Gen,_Num],[_Kas]),
       [pron([per],Form),Pron]], SemPron) :-
    !, sem([pron([per],Form),Pron],SemPron).

sem([np([DefP,3,Gen,Num],[Kas]),
      [pron([DefP],[Gen,Num,Kas]),Pron]], SemNP) :-
    (DefP = rezi; DefP = refl; DefP = qu),
    !, sem([pron([DefP],[Gen,Num,Kas]),Pron], SemNP).

sem([np([Def,3,Gen,Num],[Kas]),
      [pron([DefP],[Gen,Num,Kas]),Pron]], SemNP) :-
    DefP = rel, Def = rel(Gen,Num),
    !, sem([pron([DefP],[Gen,Num,Kas]),Pron], SemNP).

% NP = POSS+RN  
sem([np([def(Gen2,Num2),3,Gen,Num],[Kas]),
     [poss([def(Gen2,Num2)],[Gen,Num,Kas]),Poss],
     [rn([Gen],[Num,Kas]),RN]], SemNP) :-
    !, sem([poss([def(Gen2,Num2)],
                 [Gen,Num,Kas]),Poss], SemPoss),
    sem([rn([Gen],[Num,Kas]),RN], SemRN),
    normalize(SemPoss * SemRN, SemNP).

% NP = Zahl km (TODO: testen)
sem([np([_,3,_,_],[_]),
       [det([def],_),[Zahlatom]],[n(_,_),[km]]],
    lam(P,drs([],[])+P*Zahl)) :-  
    anzahl(Zahlatom,Zahl), !. % Ausnahme: Maß

% NP = DET RN NPgen
sem([np([_,3,_,_],[_]),
       Det,[rn(Art,Form),RN],NPgen], Sem) :-
    NPgen = [np([_,3,_,_],[gen])|_],
    !, sem(Det,SemDet),   
    sem([rn(Art,Form),RN],SemRN),
    sem(NPgen,SemNPgen),
    SemN = lam(X,SemNPgen*lam(Y,(SemRN*X)*Y)),
    normalize(SemDet*SemN,Sem).

% S = NP+VI 
sem([s([_Def],[_,_,vz]), NP, VI], SemS) :-
    NP = [np(_,_)|_], VI = [v([nom],_)|_],
    !, sem(NP,SemNP),
    sem(VI,SemVI),
    Sem = SemNP * SemVI,
    normalize(Sem,SemS).

sem([s([_Def],[_,_,vz]),
      NP1, [v([nom,akk],[3,Num,_,ind]),V], NP2], SemS)
:-  NP1 = [np([_,3,_,_],[Kas1])|_], 
    NP2 = [np([Def2,3,_,_],[_])|_],
    !, sem(NP1,SemNP1), sem(NP2,SemNP2),
    sem([v([nom,akk],[3,Num,_,ind]),V], SemV),
    (Kas1 = nom,
        ( (Def2 = rezi ; Def2 = refl) 
        -> Sem = SemNP1 * (SemNP2 * SemV)
        ; Sem = SemNP1 * lam(X,SemNP2 *
                            lam(Y,SemV * X * Y)))
    ; Kas1 = akk, % ausser: rezi, refl-Pronomen
        Sem = SemNP1 * lam(Y,SemNP2 *
                          lam(X,SemV * X * Y))
    ), normalize(Sem,SemS).

sem([s([rel(GenR,NumR)],[Tmp,Mod,vl]),
      NP1, NP2, [v([nom,akk],[3,Num,Tmp,Mod]),V]],
    SemS)
:-
    NP1 = [np([rel(GenR,NumR),3,GenR,_],[Kas1])|_], 
    NP2 = [np([Def2,3,_,_],[_])|_],
    !, sem(NP1,SemNP1), sem(NP2,SemNP2), 
    sem([v([nom,akk],[3,Num,Tmp,Mod]),V], SemV),
    ( Kas1 = nom, ((Def2 = refl ; Def2 = rezi)
       -> Sem = SemNP1 * (SemNP2 * SemV)
       ; Sem = 
       SemNP1 * lam(X,SemNP2 * lam(Y,SemV * X * Y)))
    ; Kas1 = akk, Sem =
      SemNP1 * lam(Y,SemNP2 * lam(X,SemV * X * Y))
    ),
    normalize(Sem,SemS).

% NP = DET N RelS
sem([np([_,3,_,_],[_]),
       Det,[n(Art,Form),N],RelS], Sem) :-
    RelS = [s([rel(_,_)],_)|_],
    !, sem(Det,SemDet),   
    sem([n(Art,Form),N],SemN),
    sem(RelS,SemRelS), 
    SemNS = lam(X, drs([],[]) + SemN*X + SemRelS*X),
    normalize(SemDet*SemNS,Sem).

sem([np([_,3,_,_],[_]),
       Det,[rn(Art,Form),RN],NPgen,RelS], Sem) :-
    RelS = [s([rel(_,_)],_)|_],
    NPgen = [np(_,[gen])|_],
    !, sem(Det,SemDet),   
    sem([rn(Art,Form),RN],SemRN),
    sem(NPgen,SemNPgen),
    sem(RelS,SemRelS), 
    SemN = lam(X,drs([],[]) + SemNPgen*lam(Y,(SemRN*X)*Y + SemRelS*X)),
    normalize(SemDet*SemN,Sem).

% D = S D
sem([d([def]),S,D], Sem) :-
    !, sem(S,SemS),
    sem(D,SemD),
    ( debugging(resolve)
    -> resolveDrs:resolve(SemS,SemD,Sem) % später
    ; normalize(SemS + SemD, Sem)
    ).

% D = [] (Diskursende)
sem([d([def])], drs([],[])) :- !.

normalize(Term,Reduced):-
    lambdaDrs:normalize(Term,Reduced).


% type(+TypKontext,+Drs,-DrsGetypt,-Typ) :-
type(Kontext,Drs,DrsGetypt,Typ) :-
    typeDrs:type(Kontext,Drs,DrsGetypt,Typ).

