% type(+Kontext,+LamTerm,-LamTermGetypt,-Typ)

% Lambda terms:
type([V:Ty|Kontext],X,X,Typ) :-
    var(X),
    !, ( X == V -> Typ = Ty ; type(Kontext,X,X,Typ) ).
type([],X,_,_) :-
    var(X), !, fail.
type(Kontext,(R * S),(RTy * STy),Typ) :-
    !, type(Kontext,S,STy,ArgT),
    type(Kontext,R,RTy,FunT),
    (unify_with_occurs_check(FunT,(ArgT -> Typ))
    -> true  % write errors to output-file?
    ; % format('\nFehler: Argument ~w hat Typ ~w',[S,ArgT]),
      % format('\n        Funktion ~w hat Typ ~w',[R,FunT]),
      fail).
type(Kontext,lam(X,Term),lam(X:Ty,TermTy),Typ) :-
    var(X),
    !, type([X:Ty|Kontext],Term,TermTy,ResultTy),
    Typ = (Ty -> ResultTy).
type(Kontext,lam((X,Y),Term),lam((X,Y):Ty,TermTy),Typ) :-
    !, type([X:TyX,Y:TyY|Kontext],Term,TermTy,ResultTy),
    Ty = (TyX * TyY), 
    Typ = (Ty -> ResultTy).
% Boolean combinations of formulas:
type(Kontext,neg(Fml),neg(FmlTy),t) :-
    !, type(Kontext,Fml,FmlTy,t).
type(Kontext,(Fml1 & Fml2),(Fml1Ty & Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 \/ Fml2),(Fml1Ty \/ Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 => Fml2),(Fml1Ty => Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 <=> Fml2),(Fml1Ty <=> Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 

type(_,true,true,t) :- !.  % Bereich: true=unbeschränkt

% Verallgemeinerte Quantoren Qu(Var,Restriction,Body) 
type(K,ex(X,R,Term),ex(X:Ty,RTy,TermTy),t) :-
    var(X), !,
    type(K,lam(X,R),lam(X:Ty,RTy),(Ty -> t)),
    type(K,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(K,all(X,R,Term),all(X:Ty,RTy,TermTy),t) :-
    var(X), !,
    type(K,lam(X,R),lam(X:Ty,RTy),(Ty -> t)),
    type(K,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(K,most(X,R,Term),most(X:Ty,RTy,TermTy),t) :-
    var(X), !,
    type(K,lam(X,R),lam(X:Ty,RTy),(Ty -> t)),
    type(K,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).

type(K,iota(X,R,Term),iota(X:Ty,RTy,TermTy),t) :-
    var(X), !,
    type(K,lam(X,R),lam(X:Ty,RTy),(Ty -> t)),
    type(K,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).

% Paar-Quantoren Qu((Var,Var),Restriction,Body) 
type(K,all((X,Y),R,Term),all((X,Y):Ty,RTy,TermTy),t) :-
    !, type(K,lam((X,Y),R),lam((X,Y):Ty,RTy),(Ty -> t)),
    type(K,lam((X,Y),Term),lam((X,Y):Ty,TermTy),(Ty -> t)).
type(K,ex((X,Y),R,Term),ex((X,Y):Ty,RTy,TermTy),t) :-
    !, type(K,lam((X,Y),R),lam((X,Y):Ty,RTy),(Ty -> t)),
    type(K,lam((X,Y),Term),lam((X,Y):Ty,TermTy),(Ty -> t)).
type(K,most((X,Y),R,Term),most((X,Y):Ty,RTy,TermTy),t) :-
    !, type(K,lam((X,Y),R),lam((X,Y):Ty,RTy),(Ty -> t)),
    type(K,lam((X,Y),Term),lam((X,Y):Ty,TermTy),(Ty -> t)).

type(K,iota((X,Y),R,Term),iota((X,Y):Ty,RTy,TermTy),t) :-
    !, type(K,lam((X,Y),R),lam((X,Y):Ty,RTy),(Ty -> t)),
    type(K,lam((X,Y),Term),lam((X,Y):Ty,TermTy),(Ty -> t)).

type(K,anzahl(X,Fml,N),anzahl(X:Ty,FmlTy,NTy),t) :-
    !, type(K,lam(X,Fml),lam(X:Ty,FmlTy),(Ty -> t)),
    type(K,N,NTy,n).  % Typ n := number

% Questions: unary/binary Wh(Vars,Restriction,Formula)
type(K,qu(X,R,Term),qu(X:Ty,RTy,TermTy),list(Ty)) :- 
    var(X), !, type(K,lam(X,R),lam(X:Ty,RTy),(Ty -> t)),
    type(K,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(K,qu((X,Y),R,Tm),qu((X,Y):Ty,RTy,TmTy),list(Ty)) :- 
    !, type(K,lam((X,Y),R),lam((X,Y):Ty,RTy),(Ty -> t)),
    type(K,lam((X,Y),Tm),lam((X,Y):Ty,TmTy),(Ty -> t)).

% Algebraic and atomic terms:
type(_Kontext,Modul:Term,Modul:Term,Typ) :- 
    !, Typ = t.         % Grundaussagen der Datenbank
type(_Kontext,N,N,Typ) :-
    number(N), !, Typ = n.
type(Kontext,(X,Y),(XTy,YTy),Typ) :- % Variablentupel 
    !, type(Kontext,X,XTy,TyX),
    type(Kontext,Y,YTy,TyY),
    Typ = (TyX * TyY).
type(Kontext,Term,Term,Typ) :- 
    Term =.. [F|Args],
    types(Kontext,Args,[],Typen),
    ( Typen = [],         TypF = Typ
    ; Typen = [T],        TypF = (T -> Typ)
    ; Typen = [T1,T2],    TypF = (T1*T2 -> Typ) ),
    functor(Term,F,Arity),
    (setof(Ty,datenbank:type(F/Arity,Ty),TypDecs)
    -> member(TypF,TypDecs) /* --- keine Fehlermeldung
       (\+ \+ member(TypF,TypDecs) % F hat passende Typen
       ->  member(TypF,TypDecs)    % wähle einen
       ; format('\nFehler: datenbank:~w hat Typen ~w, ',
                [F/Arity,TypDecs]),
           format('\nKontext erwartet ~w: ~w\n',
                  [F/Arity,TypF]), fail) ---- */
    ; format('\ndatenbank:~w hat keinen Typ\n',[F/Arity]),
        fail
    ).
% Typing a list of arguments
types(Kontext,[A|Args],Acc,TypArgs) :-
    type(Kontext,A,_,TypA),
    types(Kontext,Args,[TypA|Acc],TypArgs).
types(_Kontext,[],Typen,TypArgs) :-
    reverse(Typen,TypArgs).

