:- module(typeDrs,[]). % nicht exportiern, da man ggf. type.pl geladen hat

% Typisierung von Lambda-DRSen in einem Typkontext:    
% type(+Kontext, +Term, -GetyptTerm,-Typ).
% var
type(Kontext,X,X,Typ):-
    var(X), !,
    type_var(Kontext,X,X,Typ).

% drs1 
type(Kontext,drs([],Fs),drs([],FsT),([],t)):-
    !, type(Kontext,Fs,FsT,t).
% drs2 
type(Kontext,drs([X|R],Fs),drs([(X:Xtyp)|RT],FsT),([Xtyp|Rtyp],t)):-
    var(X), !, 
    type([X:Xtyp|Kontext],drs(R,Fs),drs(RT,FsT),(Rtyp,t)).

% merge (+)
type(Kontext,(D1 + D2),(D1T + D2T),((Typ1+Typ2),t)) :-
    !, type([D2:(Typ2,t)|Kontext],D1,D1T,(Typ1,t)),
    type([D1:(Typ1,t)|Kontext],D2,D2T,(Typ2,t)).

% app
type(Kontext,(D1 * D2),(D1T * D2T),Typ) :-
    !, type(Kontext,D1,D1T,Fkt),
    type(Kontext,D2,D2T,Arg),
    (unify_with_occurs_check(Fkt,(Arg -> Typ))
    -> true
    ; formatn('Typfehler: Argument ~W hat Typ ~W',[D2T,Arg]),
      formatn('\n           Funktion ~W hat Typ ~W\n',[D1T,Fkt]),
      fail).
% abs
type(Kontext,lam(X,D),lam(X:Xtyp,DT),Typ) :-
    !, type([X:Xtyp|Kontext],D,DT,Dtyp),
    Typ = (Xtyp -> Dtyp).

% fmls
type(Kontext,[F|Fs],[FT|FsT],t):-
    !,type(Kontext,F,FT,t),
    type(Kontext,Fs,FsT,t).

type(_Kontext,[],[],t) :-!.

% spezielle Formeln:
type(Kontext,anzahl(X,Fml,N),anzahl(X:Ty,FmlTy,NTy),t) :-
    !, type(Kontext,lam(X,Fml),lam(X:Ty,FmlTy),(Ty -> t)),
    type(Kontext,N,NTy,n).

% anaphern  
type(Kontext,anp(X,G,N),anp(X:Xtyp,G,N),t):-
    !, type(Kontext,X,X,Xtyp).
type(Kontext,anposs(X,G,N),anposs(X:Xtyp,G,N),t):-  % HL
    !, type(Kontext,X,X,Xtyp).

% antecedent  
type(Kontext,ant(X,G,N),ant(X:Xtyp,G,N),t):-
    !, type(Kontext,X,X,Xtyp).

% eqn
type(Kontext,eq(T1,T2),eq(T1T,T2T),t):-
    !, type(Kontext,T1,T1T,Typ),
    type(Kontext,T2,T2T,Typ).

% neg
type(Kontext,neg(D),neg(DT),t) :-
    !, type(Kontext,D,DT,(_Typ,t)).

% disj
type(Kontext,or(D1,D2),or(D1T,D2T),t) :-
    !, type(Kontext,D1,D1T,(_Typ1,t)),
    type(Kontext,D2,D2T,(_Typ2,t)).

% &  Konjunction of DRSs or formulas??? HL 1/14 - bug from unchanged parts of sem/2 !
type(Kontext,(D1 & D2),(D1T & D2T),t) :-
    !, type(Kontext,D1,D1T,(_Typ1,t)),
    type(Kontext,D2,D2T,(_Typ2,t)).

% impl
type(Kontext,(D1 => D2),(D1T => D2T),t) :-
    !, type(Kontext,D1,D1T,(Typ1,t)),
    type([D1:(Typ1,t)|Kontext],D2,D2T,(_Typ2,t)).

% rel + const
type(Kontext,Term,Term,Typ) :-
    Term =.. [F|Args],
    types(Kontext,Args,[],Typen),
    ( Typen = [], TypF = Typ          
    ; Typen = [T], TypF = (T -> Typ)
    ; Typen = [T1,T2], TypF = (T1*T2 -> Typ)
    ; Typen = [T1,T2,T3], TypF = (T1*T2*T3 -> Typ)
    ),
    functor(Term,F,Arity),
    (setof(Ty,datenbank:type(F/Arity,Ty),TypDecs)
    ->	member(TypF,TypDecs)
    ; formatn('\ndatenbank:~W hat keinen Typ\n',[F/Arity]),
        fail
    ).    


% types
types(Kontext,[A|Args],Acc,TypArgs) :-
    !,type(Kontext,A,_,TypA),
    types(Kontext,Args,[TypA|Acc],TypArgs).
types(_Kontext,[],Typen,TypArgs) :-
    !,reverse(Typen,TypArgs).

% var1,2 
type_var([Y:Yty|Kontext],X,X,Typ) :-
    var(Y), !,
    ( X == Y -> Typ = Yty
    ; type_var(Kontext,X,X,Typ) ).
% var4,5 
type_var([drs([Y|R],Fmls):([Ytyp|Rtyp],t)|Kontext],X,X,Typ) :-
    var(Y), !, ( X == Y -> Typ = Ytyp
    ; type_var([drs(R,Fmls):(Rtyp,t)|Kontext],X,X,Typ) ).
% var3 
type_var([drs([],_Fmls):([],t)|Kontext],X,X,Typ) :-
    !, type_var(Kontext,X,X,Typ).
% var6 
type_var([(D1 + D2):((Typ1+Typ2),t)|Kontext],X,X,Typ) :-
    !, type_var([D1:(Typ1,t), D2:(Typ2,t)|Kontext],X,X,Typ).
% var7 
type_var([(D1 * _D2):(_Ty,t)|Kontext],X,X,Typ) :-
    var_function(D1), !, % var(D1), !, % Fuer PossPron auch RN*X*Y : DrsType ignorieren!
    type_var(Kontext,X,X,Typ).
% var0
type_var([],X,_,_) :-
    !, formatn('\nTypfehler: keine Typannahme für freie Variable ~W im Kontext.\n',[X]), fail.
% VAR_fail (naechster Kontext: alles andere)
type_var(Kontext,X,_,_) :-
    !, formatn('\nTypfehler: Kontext ~w blockiert Typisierung von ~w !\n',[Kontext,X]), fail.

var_function(X) :-
    var(X), !.
var_function((D * _)) :-
    (var(D) -> true ; var_function(D)).

/*
% Anfrage
type(Term):-
	type([],Term,Getypt,Typ),
	write('\nKontext\t[]\n'),
        formatn('\nGetypt\t~w\n',[Getypt]),
	formatn('\nTyp\t~w\n',[Typ]).

type(Kontext,Term):-
	type(Kontext,Term,Getypt,Typ),
       	formatn('\nKontext\t~w\n',[Kontext]),
        formatn('\nGetypt\t~w\n',[Getypt]),
	formatn('\nTyp\t~w\n',[Typ]).
*/


/* ---------------- CL-II-Version fuer Terme statt DRSen ---------------

% type(+Kontext,+LamTerm,-LamTermGetypt,-Typ)

type([(V,Ty)|Kontext],X,X,Typ) :-
    (var(X) ; number(X)), !,
    ( X == V -> Typ = Ty ; type(Kontext,X,X,Typ) ).
type([],X,_,_) :-
    var(X), !, fail.
type([],N,N,Typ) :-
    number(N), !, Typ = n.
type(Kontext,(R * S),(RTy * STy),Typ) :-
    !, type(Kontext,S,STy,Arg),
    type(Kontext,R,RTy,FunT),
    datenbank:subtype(Arg,ArgT),
    (unify_with_occurs_check(FunT,(ArgT -> Typ))
    -> true
    ; format('Typfehler: Argument ~w hat Typ ~w',[S,Arg]),
      format('\n           Funktion ~w hat Typ ~w\n',[R,FunT]),
      fail).
type(Kontext,lam(X,Term),lam(X:Ty,TermTy),Typ) :-
    !, type([(X,Ty)|Kontext],Term,TermTy,ResultTy),
    Typ = (Ty -> ResultTy).
type(Kontext,neg(Fml),neg(FmlTy),t) :-
    !, type(Kontext,Fml,FmlTy,t).
type(Kontext,(Fml1 & Fml2),(Fml1Ty & Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 \/ Fml2),(Fml1Ty \/ Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 => Fml2),(Fml1Ty => Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,(Fml1 <=> Fml2),(Fml1Ty <=> Fml2Ty),t) :-
    !, type(Kontext,Fml1,Fml1Ty,t),
    type(Kontext,Fml2,Fml2Ty,t). 
type(Kontext,ex(X,Term),ex(X:Ty,TermTy),t) :-
    !, type(Kontext,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(Kontext,all(X,Term),all(X:Ty,TermTy),t) :-
    !, type(Kontext,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(Kontext,anzahl(X,Fml,N),anzahl(X:Ty,FmlTy,NTy),t) :-
    !, type(Kontext,lam(X,Fml),lam(X:Ty,FmlTy),(Ty -> t)),
    type(Kontext,N,NTy,n).
type(Kontext,qu(X,Term),qu(X:Ty,TermTy),list(Ty)) :- 
    !, type(Kontext,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(Kontext,qu_anzahl(X,Term),qu_anzahl(X:Ty,TermTy),n) :- 
    !, type(Kontext,lam(X,Term),lam(X:Ty,TermTy),(Ty -> t)).
type(_Kontext,Modul:Term,Modul:Term,Typ) :- 
    !, Typ = t.
type(Kontext,Term,Term,Typ) :- % TermTy=Term ??
    Term =.. [F|Args],
    types(Kontext,Args,[],Typen),
    ( Typen = [], TypF = Typ
    ; Typen = [T], TypF = (T -> Typ)
    ; Typen = [T1,T2], TypF = (T1*T2 -> Typ)
    ; Typen = [T1,T2,T3], TypF = (T1*T2*T3 -> Typ)
    ),
    functor(Term,F,Arity),
    (datenbank:type(F/Arity,TypDec)
    -> (datenbank:subtype(TypDec,TypF)
       -> true
       ; format('Typfehler: datenbank:~w hat Typ ~w',[F/Arity,TypDec]),
           format('\n           Kontext erwartet Typ ~w\n',[TypF]),
           fail
       )
    ; format('\ndatenbank:~w hat keinen Typ\n',[F/Arity]),
        fail).

types(Kontext,[A|Args],Acc,TypArgs) :-
    type(Kontext,A,_,TypA),
    types(Kontext,Args,[TypA|Acc],TypArgs).
types(_Kontext,[],Typen,TypArgs) :-
    reverse(Typen,TypArgs).

*/
